/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.jboss4.nodes;

import javax.swing.Action;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.actions.SystemAction;

/**
 * Node which describes Servlet.
 *
 * @author Michal Mocnak
 */
public class JBServletNode extends AbstractNode {
    public JBServletNode(String name) {
        super(Children.LEAF);
        setDisplayName(name);
        setIconBaseWithExtension("org/netbeans/modules/j2ee/jboss4/resources/Object.gif"); // NOI18N
    }
    
    public Action[] getActions(boolean context){
        return new SystemAction[] {};
    }
}
