
package testPackage;


/**
 * This is the local interface for SalesTaxCodeTbl enterprise bean.
 */
public interface SalesTaxCodeTblLocal extends javax.ejb.EJBLocalObject, testPackage.SalesTaxCodeTblLocalBusiness {
    
    
}
