/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.dd.impl.web.metadata;

import java.util.List;
import org.netbeans.modules.j2ee.dd.api.web.model.FilterInfo;
import org.openide.util.Exceptions;

/**
 * @author Petr Slechta
 */
public abstract class FilterInfoAccessor {

    private static volatile FilterInfoAccessor accessor;

    public static void setDefault(FilterInfoAccessor accessor) {
        if (FilterInfoAccessor.accessor != null) {
            throw new IllegalStateException("Already initialized accessor"); // NOI18N
        }
        FilterInfoAccessor.accessor = accessor;
    }

    public static FilterInfoAccessor getDefault() {
        if (accessor != null) {
            return accessor;
        }

        Class c = FilterInfo.class;
        try {
            Class.forName(c.getName(), true, FilterInfoAccessor.class.getClassLoader());
        } catch (ClassNotFoundException cnf) {
            Exceptions.printStackTrace(cnf);
        }

        return accessor;
    }

    public abstract FilterInfo createFilterInfo(String name, String filterClass, List<String> urlPatterns);
}
