/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.el;

import org.netbeans.api.java.source.CompilationInfo;
import org.netbeans.modules.web.el.spi.ResolverContext;
import org.openide.filesystems.FileObject;

/**
 *
 * @author marekfukala
 */
public class CompilationContext {
    
    private final FileObject file;
    private final CompilationInfo info;
    private final ResolverContext context = new ResolverContext();
    private CompilationCache cache;

    private CompilationContext(FileObject file, CompilationInfo info) {
        this.file = file;
        this.info = info;
    }
    
    public static CompilationContext create(FileObject file, CompilationInfo info) {
        return new CompilationContext(file, info);
    }

    public FileObject file() {
        return file;
    }

    public CompilationInfo info() {
        return info;
    }

    public ResolverContext context() {
        return context;
    }
    
    public synchronized CompilationCache cache() {
        if(cache == null) {
            cache = new CompilationCache();
        }
        return cache;
    }
    
}
