/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.beans.navigation;

import javax.lang.model.element.TypeElement;

import org.netbeans.api.java.source.CompilationController;
import org.netbeans.api.java.source.CompilationInfo;
import org.openide.filesystems.FileObject;


/**
 * @author ads
 *
 */
class TypeTreeNode extends InjectableTreeNode<TypeElement> {

    private static final long serialVersionUID = 5945151445570825042L;

    TypeTreeNode(FileObject fileObject, TypeElement typeElement, boolean disabled,
        CompilationInfo compilationInfo) 
    {
        super(fileObject, typeElement, null, disabled, compilationInfo);
    }
    
    boolean isAssignableFrom( TypeElement element , 
            CompilationController controller)
    {
        TypeElement typeElement = getElementHandle().resolve(controller);
        if ( typeElement ==null ){
            return true;
        }
        return controller.getTypes().isAssignable( element.asType(), 
                typeElement.asType());
    }
    
    boolean isAssignable( TypeElement element , CompilationController controller ){
        TypeElement typeElement = getElementHandle().resolve(controller);
        if ( typeElement ==null ){
            return false;
        }
        return controller.getTypes().isAssignable( typeElement.asType(), 
                element.asType());
    }
    
}
