/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package test.jms;

import javax.jms.JMSDestinationDefinition;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
@JMSDestinationDefinition(className = "javax.ejb.Topic", name = "topicInOneClass")
public class OneDefinitionClass {

}
