/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.jsf.refactoring;

import junit.framework.TestCase;
import org.openide.filesystems.FileObject;

/**
 *
 * @author petr
 */
public class JSFRefactoringUtilsTest extends TestCase {
    
    public JSFRefactoringUtilsTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testContainsRenamingPackage() {
        assertTrue(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "a.b.c", false));
        assertFalse(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "a.b", false));
        assertTrue(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "a.b", true));
        assertTrue(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "a", true));
        assertTrue(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "a.b.c", true));
        assertFalse(JSFRefactoringUtils.containsRenamingPackage("a.b.c.class", "x.y.z", false));
        
        
    }
    
}
