/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package testGenerateJavaEE60;

import javax.ejb.Singleton;

/**
 *
 * @author {user}
 */
@Singleton
public class TestSingleton {

    // Add business logic below. (Right-click in editor and choose
    // "Insert Code > Add Business Method")

}
