
package ejb;

import javax.ejb.EJBObject;


/**
 * This is the remote interface for SessionTest2 enterprise bean.
 */
public interface SessionTest2Remote extends EJBObject, SessionTest2RemoteBusiness {


}
