

/**CC 
@|
@media
@media
*/



/**CC 
@|
@charset
@charset
*/

@media page{

    root {

/**CC
c|
color
color:
*/

    
/**CC
background: |
background url
background: url
*/


/**CC
color: |
color green
color: green
*/

      
    }

}

