/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.websvc.manager.model;

import java.beans.DefaultPersistenceDelegate;
import java.beans.Encoder;

/**
 *
 * @author  david
 */
public class WebServiceDataPersistenceDelegate extends DefaultPersistenceDelegate {
    
    /** Creates a new instance of WebServiceDataPersistenceDelegate */
    public WebServiceDataPersistenceDelegate() {
    }
    
    /**
     * Overrideing PersistenceDelegate to clean up any unwanted classes trying to be written.
     */
    public void writeObject(Object oldInstance, Encoder out) {
        if(oldInstance instanceof javax.xml.namespace.QName) {
            return;
//        } else if(oldInstance instanceof com.sun.xml.rpc.wsdl.document.soap.SOAPStyle) {
//            return;
//        } else if(oldInstance instanceof com.sun.xml.rpc.wsdl.document.soap.SOAPUse) {
//            return;
        }else {
            super.writeObject(oldInstance,out);
        }
        
    }
    
    
}
