/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.websvc.api.jaxws.wsdlmodel;

import org.xml.sax.SAXParseException;

/**
 *
 * @author mkuchtiak
 */
public interface WsdlErrorHandler {
    
    /** callbac method : warning
     * @param ex exception describing a warning
     * @throws AbortException to enable abort the parsing process
    */ 
    public void warning(SAXParseException ex) throws AbortException ;
  
    /** callbac method : info
     * @param ex exception describing an information
    */ 
    public void info(SAXParseException ex);
    
    /** callbac method : fatalError
     * @param ex exception describing a fatal error
     * @throws AbortException to enable abort the parsing process
    */ 
    public void fatalError(SAXParseException ex) throws AbortException ;
    
    /** callbac method : error
     * @param ex exception describing an error
     * @throws AbortException to enable abort the parsing process
    */ 
    public void error(SAXParseException ex) throws AbortException ;
    
    /** Exception to enable abort the parse process
     */ 
    public static class AbortException extends Exception {
        public AbortException() {
            super();
        }
        public AbortException(String message) {
            super(message);
        }
    }

}
