/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.jsf.api.facesmodel;

import java.util.List;
import org.netbeans.modules.web.jsf.impl.facesmodel.JSFConfigQNames;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public interface FlowCall extends IdentifiableElement {

    /**
     * Property name of &lt;faces-flow-reference&gt; element.
     */
    static final String FLOW_REFERENCE = JSFConfigQNames.FLOW_REFERENCE.getLocalName();

    /**
     * Property name of &lt;outbound-parameter&gt; element.
     */
    static final String OUTBOUND_PARAMETER = JSFConfigQNames.OUTBOUND_PARAMETER.getLocalName();

    List<FlowCallFacesFlowReference> getFacesFlowReferences();
    void addFacesFlowReference(FlowCallFacesFlowReference facesFlowReference);
    void removeFacesFlowReference(FlowCallFacesFlowReference facesFlowReference);


    List<FlowCallOutboundParameter> getOutboundParameters();
    void addOutboundParameter(FlowCallOutboundParameter outboundParameter);
    void removeOutboundParameter(FlowCallOutboundParameter outboundParameter);
}
