
package test;


/**
 * This is the local interface for TestingSession enterprise bean.
 */
public interface TestingSessionLocal extends javax.ejb.EJBLocalObject, test.TestingSessionLocalBusiness {
    
    
}
