<%@page contentType="text/html" pageEncoding="UTF-8"%>
<html>
    <head>
        <title></title>
        </head>
        <body>
    <a href="#" onclick="newCustomerWindow(${param.cust_no});">Edit</a>
    <img src="/images/icon.gif" onclick="imageClicked()" alt="img"/>

</html>