/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.jsf.editor.facelets;

import org.netbeans.modules.web.jsfapi.api.Function;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class FunctionImpl implements Function {

    private String name;
    private String signature;
    private String desc;

    public FunctionImpl(String name, String signature, String desc) {
        this.name = name;
        this.signature = signature;
        this.desc = desc;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getSignature() {
        return signature;
    }

    @Override
    public String getDescription() {
        return desc;
    }

    @Override
    public String toString() {
        return "Function[name=" + getName() + ", signature" + getSignature() + "]";
    }
}
