/*
 * TmpImplTest.java
 * JUnit based test
 *
 * Created on July 26, 2005, 1:10 PM
 */

/**
 *
 * @author jungi
 */
public class TmpImplTest extends junit.framework.TestCase {
    
    public TmpImplTest(String testName) {
        super(testName);
    }
    
}
