/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.deployment.common.api;

/**
 * Represents a message destination
 *
 * @author Libor Kotouc
 *
 * @since 1.25
 */
public interface MessageDestination {
    
    /**
     * Message destination type
     */
    public enum Type {
        /**
         * Queue
         */ 
        QUEUE, 
        
        /**
         * Topic
         */
        TOPIC
    };
    
    /**
     * Returns the name
     *
     * @return the name
     */
    public String getName();
    
    /**
     * Returns the type
     * 
     * @return the type
     */
    public Type getType();
}
