<html>
    <body>        
        <%                
        if(true) {
            a = b ;
        }
        %>
        <%
  for (int i=0; i < servers.length; i++)
  {
%>
<%
  }
%>
    </body>
</html>