package org.wa59055;

import junit.framework.*;

public class Class1Test extends TestCase {
    
    public Class1Test(String testName) {
        super(testName);
    }

    /**
     * Test of method1 method, of class org.wa59055.Class1.
     */
    public void testMethod1() {
        System.out.println("method1");
        
        Class1 instance = new Class1();
        
        instance.method1();
        
        // TODO review the generated test code and remove the default call to fail.
        fail("The test case is a prototype.");
    }
    
}
