/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.websvc.wsitconf.api;

import java.util.logging.Logger;
import org.netbeans.modules.websvc.wsitconf.util.DefaultSettings;

/**
 *
 * @author Martin Grebac
 */
public final class DevDefaultsProvider extends Object {

    private static final Logger logger = Logger.getLogger(DevDefaultsProvider.class.getName());

    private static DevDefaultsProvider instance;
  
    private DevDefaultsProvider() { }

    public static synchronized DevDefaultsProvider getDefault() {
        if (instance == null) {
            instance = new DevDefaultsProvider();
        }
        return instance;
    }

     /**
      * Sets up the server environment by creating proper keystores if required, and 
      * filling proper keys into them
      */
    public final synchronized void fillDefaultsToServer(String serverID) {
        DefaultSettings.fillDefaultsToServer(serverID);
    }
    
}
