body {
    margin: 0px;
}

#top {
    background-color: #036fab;
    color: white;
    padding: 10px;
    margin: 0px 0px 10px 0px;
}

#bottom {
    background-color: #c2dfef;
    padding: 10px;
}

#left {
    background-color: #ece3a5;
    padding: 10px;
    width: 150px;
    vertical-align: top;
}

#right {
    background-color: #ece3a5;
    padding: 10px;
    width: 150px;
    vertical-align: top;
}

#content {
    background-color: #dddddd;
    padding: 10px;
        vertical-align: top;
}

#top a:link, #top a:visited {
  color: white;
  font-weight : bold;
  text-decoration: none;
}

#top a:link:hover, #top a:visited:hover  {
  color: black;
  font-weight : bold;
  text-decoration : underline;
}