<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8"%>

<%-- completion for HTML tags --%>
<%--CC
<tab|
...
<table
--%>

<%-- completion for HTML values --%>
<%--CC
<p dir="r|
...
<p dir="rtl
--%>

<%-- completion for HTML entities --%>
<%--CC
&quo|
...
&quot;
--%>

<%-- completion for body HTML tag --%>
<%--CC
<bod|
...
<body
--%>

<%-- completion for body HTML endtag --%>
<%--CC
<body></|
...
<body></body>
--%>

<%-- completion for body HTML endtag --%>
<%--CC
<table></ta|
...
<table></table>
--%>

<html>
    <head><title>HTML Code Completion Page</title></head>
    <body>
        <h1>HTML Code Completion Page</h1>
    </body>
</html>
