/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.tomcat5.progress;

import javax.enterprise.deploy.spi.Target;
import javax.enterprise.deploy.spi.TargetModuleID;

/**
 *
 * @author Petr Hejl
 */
public class TargetModuleIDImpl implements TargetModuleID {

    private final String targetName;
    
    private final String targetDescription;
    
    private final String moduleID;
    
    private final String url;
    
    public TargetModuleIDImpl(String targetName, String targetDescription,
            String moduleID, String url) {
        
        this.targetName = targetName;
        this.targetDescription = targetDescription;
        this.moduleID = moduleID;
        this.url = url;
    }

    @Override
    public TargetModuleID[] getChildTargetModuleID() {
        return new TargetModuleID[0];
    }

    @Override
    public String getModuleID() {
        return moduleID;
    }

    @Override
    public TargetModuleID getParentTargetModuleID() {
        return null;
    }

    @Override
    public Target getTarget() {
        return new Target() {

            @Override
            public String getDescription() {
                return targetDescription;
            }

            @Override
            public String getName() {
                return targetName;
            }
        };
    }

    @Override
    public String getWebURL() {
        return url;
    }
}
