/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.jsfapi.api;

import org.netbeans.modules.web.common.api.WebPageMetadata;
import org.openide.filesystems.FileObject;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;

/**
 *
 * @author marekfukala
 */
public class JsfUtils {

    public static final String JSF_XHTML_FILE_MIMETYPE = "text/facelets"; //NOI18N

    public static boolean isFaceletsFile(FileObject fo) {
        InstanceContent ic = new InstanceContent();
        ic.add(fo);
        WebPageMetadata wpm = WebPageMetadata.getMetadata(new AbstractLookup(ic));
        if(wpm == null) {
            return false;
        }
        String mime = (String)wpm.value(WebPageMetadata.MIMETYPE);
        return JSF_XHTML_FILE_MIMETYPE.equals(mime);
    }

}
