
package test;

import javax.ejb.EJBLocalObject;


/**
 * This is the local interface for EntityB enterprise bean.
 */
public interface EntityBLocal extends EJBLocalObject, EntityBLocalBusiness {
    
    
}
