<!-- Updates and News Box for our site -->
<li id="news">
    <h2>News &amp; Updates</h2>
    <ul>
        <li>
            <h3>25th May</h3>
            <p><a href="#">See what's new with AirAirlines — explore our current press releases.</a></p>
        </li>
        <li>
            <h3>23rd May</h3>
            <p><a href="#">Find useful information on SIA stock, shareholdings, financial, and operating data for investors.</a></p>
        </li>
        <li>
            <h3>21st May</h3>
            <p><a href="#">Find a job at AirAirlines and watch your career takeoff!</a></p>
        </li>        
    </ul>
</li>
