/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.javaee.resources.api;

/**
 * Describes single JmsDestination JNDI resource.
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public interface JmsDestination extends JndiResource {

    /**
     * Description of the JmsDestination.
     * @return JmsDestination's description
     */
    String getDescription();

    /**
     * Name of the JmsDestination.
     * @return JmsDestination's name
     */
    String getName();

    /**
     * ClassName of the JmsDestination.
     * @return JmsDestination's className
     */
    String getClassName();

    /**
     * InterfaceName of the JmsDestination.
     * @return JmsDestination's interfaceName
     */
    String getInterfaceName();

    /**
     * DestinationName of the JmsDestination.
     * @return JmsDestination's destinationName
     */
    String getDestinationName();

    /**
     * ResourceAdapterName of the JmsDestination.
     * @return JmsDestination's resourceAdapterName
     */
    String getResourceAdapterName();

    /**
     * Properties of the JmsDestination.
     * @return array of JmsDestination's properties
     */
    String[] getProperties();

}
