
package ejb;

import javax.ejb.EJBLocalObject;


/**
 * This is the local interface for SessionTest2 enterprise bean.
 */
public interface SessionTest2Local extends EJBLocalObject, SessionTest2LocalBusiness {


}
