/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package wsdl;

import javax.ejb.Stateless;
import javax.jws.HandlerChain;
import javax.jws.WebService;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.soap.SOAPBinding;

/**
 *
 * @author js201828
 */

//Annotations
/**CC
@W|
WebService
@WebService
*/

/**CC
@S|
SOAPBinding
@SOAPBinding
*/

//attributes
/**CC
@HandlerChain(|)
String file
@HandlerChain(file=)
*/

//values
/**CC
@WebParam(header=|)
true
@WebParam(header=true)
*/


//values
/**CC
@WebParam(mode=|)
Mode Mode.OUT
@WebParam(mode= WebParam.Mode.OUT)
*/

//file sections
/**
@HandlerChain(file="|")
../
@HandlerChain(file="../")
 */

//file sections
/**
@HandlerChain(file="../|")
wsdl/
@HandlerChain(file="../wsdl/")
*/

/**
@WebService(wsdlLocation="")
testWSDL.wsdl
@WebService(wsdlLocation="META-INF/wsdl/testWSDL.wsdl")
*/

public interface TestWebService extends java.rmi.Remote {
    
}




