/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.websvc.core.jaxws.nodes;

/** Port children (Operation elements)
 *
 * @author mkuchtiak
 */
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.netbeans.modules.websvc.api.jaxws.wsdlmodel.WsdlOperation;
import org.netbeans.modules.websvc.api.jaxws.wsdlmodel.WsdlPort;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

public class PortChildren extends Children.Keys<WsdlOperation> {
    WsdlPort wsdlPort;
    
    public PortChildren(WsdlPort wsdlPort) {
        this.wsdlPort=wsdlPort;
    }
    
    @Override
    protected void addNotify() {
        super.addNotify();
        updateKeys();
    }
    
    @Override
    protected void removeNotify() {
        setKeys(Collections.<WsdlOperation>emptyList());
        super.removeNotify();
    }
       
    private void updateKeys() {
        List<WsdlOperation> keys =  wsdlPort.getOperations();
        if ( keys == null ){
            setKeys(Collections.<WsdlOperation>emptyList() );
        }
        else {
            Collections.sort( keys , WsdlOperationComparator.getInstance());
            setKeys(keys);
        }
        
    }

    protected Node[] createNodes(WsdlOperation key) {
        return new Node[] {new OperationNode((WsdlOperation) key)};
    }

}
