/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.groovy.grails.api;

import java.beans.PropertyChangeListener;
import javax.swing.Icon;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.openide.filesystems.FileObject;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Petr Hejl
 */
public class TestProject implements Project {

    private final FileObject directory;

    private final Lookup lookup;

    public TestProject(String name, FileObject directory) {
        this.directory = directory;
        lookup = Lookups.fixed(this, new TestProjectInformation(this, name), new GrailsProjectConfig(this));
    }

    public Lookup getLookup() {
        return lookup;
    }

    public FileObject getProjectDirectory() {
        return directory;
    }

    private static class TestProjectInformation implements ProjectInformation {

        private final Project project;

        private final String name;

        public TestProjectInformation(Project project, String name) {
            this.project = project;
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public Project getProject() {
            return project;
        }

        public String getDisplayName() {
            return name;
        }

        public Icon getIcon() {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public void addPropertyChangeListener(PropertyChangeListener listener) {

        }

        public void removePropertyChangeListener(PropertyChangeListener listener) {

        }

    }
}
