* {
    margin: 0;
    padding: 0;
}

.form_submitb{
    FONT-WEIGHT: normal; FONT-SIZE: 11px; COLOR: #FFFFFF; BORDER-TOP-STYLE: groove; FONT-STYLE: normal; FONT-FAMILY: verdana; BORDER-RIGHT-STYLE: ridge; BORDER-LEFT-STYLE: groove; BACKGROUND-COLOR: #AA2808; BORDER-BOTTOM-STYLE: ridge;
}

.form_tfield{
    FONT-WEIGHT: normal; FONT-SIZE: 11px; COLOR: #000000; BORDER-TOP-STYLE: groove; FONT-STYLE: normal; FONT-FAMILY: verdana; BORDER-RIGHT-STYLE: ridge; BORDER-LEFT-STYLE: groove; BACKGROUND-COLOR: #FFFFFF; BORDER-BOTTOM-STYLE: ridge;
}

body {
    background: #D4C792 url(images/img01.gif);
    font-family: Arial, Helvetica, sans-serif;
    font-size: 13px;
    color: #333333;
}

h1, h2, h3 {
    color: #AA2808;
}

h1 {
}

h2 {
}

h3 {
}

p, blockquote, ul, ol {
    margin-bottom: 20px;
    line-height: 1.6em;
}

p {
}

blockquote {
}

ul {
}

ol {
}

a {
    text-decoration: underline;
    color: #323B51;
}

a.data {
    text-decoration: underline;
    color: yellow;
}

a:hover {
    text-decoration: none;
    color: #FF0000;
}



/* Wrapper */

#wrapper {
}

/* Header */

#header {
    width: 710px;
    height: 96px;
    margin: 0 auto;
    background: url(images/img05.gif) no-repeat left bottom;
}

/* Logo */

#logo {
    float: left;
    padding: 20px 0 0 15px;
}

#logo h1 {
    margin: 0;
    padding: 0;
    font: italic 36px "Trebuchet MS", Arial, Helvetica, sans-serif;
}

#logo h2 {
    margin: -5px 0 0 0;
    padding: 0;
    text-transform: uppercase;
    letter-spacing: 2px;
    font-size: 10px;
    font-weight: bold;
    color: #D4C792;
}

#logo a {
    text-decoration: none;
    color: #FFFFFF;
}

/* Menu */

#menu {
    float: right;
    padding-right: 10px;
}

#menu ul {
    margin: 0;
    padding: 50px 0 0 0;
    list-style: none;
    line-height: normal;
}

#menu li {
    display: inline;
}

#menu a {
    display: block;
    float: left;
    width: 110px;
    height: 20px;
    margin: 0;
    padding: 10px 0 0 0;
    background: url(images/img02.gif);
    text-align: center;
    text-transform: uppercase;
    font-size: 11px;
    font-weight: bold;
    color: #FFFFFF;
}

#menu a:hover, #menu .active a {
    background-image: url(images/img03.gif);
    color: #AA2808;
}

/* Page */

#page {
    width: 710px;
    margin: 0 auto;
    background: #D4C792 url(images/img04.gif) repeat-y;
}

/* Content */

#content {
    float: left;
    width: 388px;
    padding: 30px 0 0 35px;
    
}

#content h1, #content h2, #content h3 {
    margin-bottom: 20px;
}

#content h1 {
    font-size: 136%;
}

#content h2 {
    font-size: 107%;
}

#content h3 {
    font: 92%;
}

#content ul, #content ol {
    list-style-position: inside;
}

#content .boxed {
    float: left;
    width: 46%;
    padding-right: 2%;
}


/* Sidebar */

#sidebar {
    float: right;
    width: 208px;
    padding: 0 35px 0 0;
}

#sidebar ul {
    margin: 0;
    padding: 0;
    list-style: none;
    line-height: normal;
}

#sidebar li {
    margin-bottom: 1px;
    padding: 20px 0;
}


#sidebar li li {
    margin: 0;
    padding: 7px 20px;
    border: none;
}

#sidebar h2 {
    padding: 5px 20px;	
    border-bottom: 1px dotted #FFFFFF;
    font-size: 100%;
    color: #D4C792;
}

#sidebar h3 {
    font-size: 77%;
    color: #D4C792;
}

#sidebar p {
    margin: 0;
    line-height: normal;
    color: #D4C792;
}

#sidebar a {
    border: none;
    text-decoration: none;
    color: #FFFFFF;
}

#sidebar a:hover {
    text-decoration: underline;
}

/* Submenu */

#submenu {
}

/* News */

#news {
}

#news a {
    font-size: 85%;
}

/* Footer */

#footer {
    width: 710px;
    height: 50px;
    margin: 0 auto;
    padding: 40px 0 0 0;
    background: url(images/img06.gif) no-repeat;
    color: #FFFFFF;
}

#footer p {
    margin: 0;
    text-align: center;
    font-size: 77%;
}

#footer a {
    text-decoration: underline;
    color: #FFFFFF;
}

#footer a:hover {
    text-decoration: none;
}
TABLE.aatable
{ 
    font-family:arial;
    font-size:10pt;
    background-color:#808080;
    width:400px;
    border-style:solid;
    border-color:#D4C792;
    border-width:2px;     
}

TH
{
    font-size:10pt;
    color:white;
    text-align:left;
}


TR
{ 
}

TD
{  
    font-size:9pt;
    background-color:#AA2808;
    color:white;
    border-style:solid;
    border-width:1px;
    text-align:left;
}

#note
{  
    font-size:7pt;    
    color:black;     
}
#UILabel
{  
    font-size:8pt;    
    color:#AA2808;     
}