
package testPackage;


/**
 * This is the local interface for OrderTbl enterprise bean.
 */
public interface OrderTblLocal extends javax.ejb.EJBLocalObject, testPackage.OrderTblLocalBusiness {
    
    
}
