/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
@OptionsPanelController.ContainerRegistration(
    id = "CPlusPlus",
    iconBase = "org/netbeans/modules/cpplite/editor/lsp/options/cnd_32.png",
    categoryName = "#OptionsCategory_Name_CPlusPlus",
    keywords = "#OptionsCategory_Keywords_CPlusPlus",
    keywordsCategory = "CPPLite",
    position = 715
)
@org.openide.util.NbBundle.Messages({"OptionsCategory_Name_CPlusPlus=C/C++", "OptionsCategory_Keywords_CPlusPlus=C C++"})
package org.netbeans.modules.cpplite.editor.lsp.options;

import org.netbeans.spi.options.OptionsPanelController;
