.ui-widget {
    font-size: 12px !important;
}
.ui-layout-north {
    z-index: 20 !important;
    overflow: visible !important;;
}
.ui-layout-north .ui-layout-unit-content {
    overflow: visible !important;
}
.ui-widget-header {
    text-align: center;
    text-transform: uppercase;
}
.ui-button {
    margin-top: 10px !important;
}
