/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.test.web;

import junit.framework.Test;

/**
 */
public class WebProjectValidationEE5 extends WebProjectValidation {

    public static final String[] TESTS = new String[]{
        "testNewWebProject", "testRedeployProject",
        "testNewJSP", "testNewJSP2", "testNewServlet", "testNewServlet2",
        "testCompileAllJSP", "testCompileJSP",
        "testCleanAndBuildProject", "testRunProject", "testRunJSP", "testViewServlet",
        "testRunServlet", "testCreateTLD", "testCreateTagHandler", "testRunTag",
        "testNewHTML", "testRunHTML", "testNewSegment", "testNewDocument",
        "testFinish"
    };

    public WebProjectValidationEE5(String name) {
        super(name);
        PROJECT_NAME = "WebJ2EE5Project"; // NOI18N
    }

    public static Test suite() {
        return createAllModulesServerSuite(Server.GLASSFISH, WebProjectValidationEE5.class, TESTS);
    }

    @Override
    protected String getEEVersion() {
        return JAVA_EE_5;
    }
}
