/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.jsf.wizards;

public class WizardProperties {

    /** Properties. */
    public static final String CONFIG_FILE = "ConfigFile"; // NOI18N
    public static final String DESCRIPTION = "Description"; // NOI18N
    public static final String NAME = "Name"; // NOI18N
    public static final String SCOPE = "Scope"; // NOI18N
    public static final String ENTITY_CLASS = "EntityClass"; //NOI18N
    public static final String JSF_FOLDER = "JSFFolder"; //NOI18N
    public static final String JSF_CLASSES_PACKAGE = "JsfClassesPackage"; //NOI18N
    public static final String JPA_CLASSES_PACKAGE = "JpaClassesPackage"; //NOI18N
    public static final String JAVA_PACKAGE_ROOT_FILE_OBJECT = "JavaPackageRootFileObject"; //NOI18N
    public static final String AJAXIFY_JSF_CRUD = "AjaxifyJsfCrud"; //NOI18N
    public static final String LOCALIZATION_BUNDLE_NAME = "localizationBundleName"; //NOI18N
    public static final String TEMPLATE_STYLE = "templateStyle"; //NOI18N

    /** Template files. */
    public static final String VIEW_TEMPLATE = "view.ftl"; // NOI18N
    public static final String LIST_TEMPLATE = "list.ftl"; // NOI18N
    public static final String CREATE_TEMPLATE = "create.ftl"; // NOI18N
    public static final String EDIT_TEMPLATE = "edit.ftl"; // NOI18N
    public static final String CONTROLLER_TEMPLATE = "controller.ftl"; // NOI18N
    public static final String PAGINATION_TEMPLATE = "PaginationHelper.ftl"; // NOI18N
    public static final String UTIL_TEMPLATE = "JsfUtil.ftl"; // NOI18N
    public static final String BUNDLE_TEMPLATE = "bundle.ftl"; // NOI18N
    public static final String BASE_TEMPLATE = "template.ftl"; // NOI18N
}
