/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.beans.xml.impl;

import java.util.List;

import org.netbeans.modules.web.beans.xml.BeanClass;
import org.netbeans.modules.web.beans.xml.BeanClassContainer;
import org.w3c.dom.Element;


/**
 * @author ads
 *
 */
abstract class ClassContainerImpl extends BaseClassContainerImpl 
    implements BeanClassContainer 
{
    
    ClassContainerImpl( WebBeansModelImpl model, Element e ) {
        super(model, e);
    }


    public void addBeanClass( BeanClass clazz ) {
        appendChild(BeanClass.CLASS, clazz);
    }

    public void addBeanClass( int index, BeanClass clazz ) {
        insertAtIndex(BeanClass.CLASS, clazz, index);
    }

    public List<BeanClass> getBeansClasses() {
        return getChildren( BeanClass.class );
    }

    public void removeBeanClass( BeanClass clazz ) {
        removeChild( BeanClass.CLASS, clazz);
    }

}
