
package test;


/**
 * This is the business interface for TestingEntity enterprise bean.
 */
public interface TestingEntityRemoteBusiness {
    public abstract java.lang.String getKey() throws java.rmi.RemoteException;
    
}
