/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.api.ejbjar;

/**
 *
 * @author Martin Adamek
 */
public final class MessageDestinationReference {

    private final String messageDestinationRefName;
    private final String messageDestinationType;
    private final String messageDestinationUsage;
    private final String messageDestinationLink;
    
    private MessageDestinationReference(String messageDestinationRefName, String messageDestinationType, String messageDestinationUsage, String messageDestinationLink) {
        this.messageDestinationRefName = messageDestinationRefName;
        this.messageDestinationType = messageDestinationType;
        this.messageDestinationUsage = messageDestinationUsage;
        this.messageDestinationLink = messageDestinationLink;
    }

    public static MessageDestinationReference create(String messageDestinationRefName, String messageDestinationType, String messageDestinationUsage, String messageDestinationLink) {
        return new MessageDestinationReference(messageDestinationRefName, messageDestinationType, messageDestinationUsage, messageDestinationLink);
    }
    
    public String getMessageDestinationRefName() {
        return messageDestinationRefName;
    }

    public String getMessageDestinationType() {
        return messageDestinationType;
    }
    
    public String getMessageDestinationUsage() {
        return messageDestinationUsage;
    }
    
    public String getMessageDestinationLink() {
        return messageDestinationLink;
    }
    
}
