/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.el.hints;

import org.netbeans.modules.csl.api.HintsProvider.HintsManager;
import org.netbeans.modules.web.el.ELLanguage;
import org.netbeans.spi.options.AdvancedOption;
import org.netbeans.spi.options.OptionsPanelController;
import org.openide.util.NbBundle;

public final class HintsAdvancedOption extends AdvancedOption {

    private OptionsPanelController panelController;

    @Override
    public String getDisplayName() {
        return NbBundle.getMessage(HintsAdvancedOption.class, "CTL_Hints_DisplayName"); // NOI18N
    }

    @Override
    public String getTooltip() {
        return NbBundle.getMessage(HintsAdvancedOption.class, "CTL_Hints_ToolTip"); // NOI18N
    }

    @Override
    public synchronized OptionsPanelController create() {
        if ( panelController == null ) {
            HintsManager manager = HintsManager.getManagerForMimeType(ELLanguage.MIME_TYPE);
            assert manager != null;
            panelController = manager.getOptionsController();
        }
        
        return panelController;
    }

    //TODO: temporary solution, this should be solved on GSF level
    public static  OptionsPanelController createStatic(){
        return new HintsAdvancedOption().create();
    }
}
