
package testPackage;


/**
 * This is the local interface for OfficeTypeCodeTbl enterprise bean.
 */
public interface OfficeTypeCodeTblLocal extends javax.ejb.EJBLocalObject, testPackage.OfficeTypeCodeTblLocalBusiness {
    
    
}
