<!-- Common Header for our site -->
<div id="header">
    <div id="logo">
        <h1><a href="index.php">Air Alliance</a></h1>
        <h2><a href="index.php">Serving Guests Since 1430</a></h2>
    </div>
    <!-- end div#logo -->
    <div id="menu">
        <ul>
            <li><a href="index.php">Home</a></li>
            <li><a href="processitinerary.php">Book</a></li>
            <li><a href="confirmreservation.php">Check</a></li>
           
        </ul>
    </div>
    <!-- end div#menu -->
</div>