
package testPackage;


/**
 * This is the local interface for MicroMarketsTbl enterprise bean.
 */
public interface MicroMarketsTblLocal extends javax.ejb.EJBLocalObject, testPackage.MicroMarketsTblLocalBusiness {
    
    
}
