<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8" import="javax.persistence.*"%>

<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title>JSP Page</title>
    </head>
    <body>
    <h1>JSP Page</h1>

<%-- Completion for all J2EE annotation names --%>
<%--CC
<% @J|
JoinColumn
<% @JoinColumn
--%>

<%-- Completion for reduced J2EE annotation names --%>

<%--CC
<% @Ta|
TableGenerator
<% @TableGenerator
--%>


<%-- Completion for all J2EE annotation attributes --%>
<%--CC
<% @ManyToOne(|
boolean optional = true
<% @ManyToOne(optional=
--%>

<%-- Completion for all J2EE annotation values --%>
<%--CC
<% @Basic(fetch=FetchType.|
FetchType LAZY
<% @Basic(fetch=FetchType.LAZY
--%>

<%-- Completion for reduced J2EE annotation values --%>
<%--CC
<% @ManyToOne(cascade=CascadeType.R|
CascadeType REMOVE
<% @ManyToOne(cascade=CascadeType.REMOVE
--%>

    </body>
</html>
