
package test;

import javax.ejb.EJBLocalObject;


/**
 * This is the local interface for EntityA enterprise bean.
 */
public interface EntityALocal extends EJBLocalObject, EntityALocalBusiness {
    
    
}
