/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.performance.j2ee.setup;

import org.netbeans.modules.performance.utilities.CommonUtilities;
import org.netbeans.modules.performance.utilities.PerformanceTestCase2;

public class J2EESetup extends PerformanceTestCase2 {

    public J2EESetup(java.lang.String testName) {
        super(testName);
    }

    public void testCloseAllDocuments() {
        CommonUtilities.closeAllDocuments();
    }

    public void testCloseMemoryToolbar() {
        CommonUtilities.closeMemoryToolbar();
    }

    public void testAddAppServer() {
        CommonUtilities.addApplicationServer();
    }

    public void testOpenTestApplications() throws Exception {
        this.openDataProjects("TestApplication");
    }

    public void testOpenTestApplication_ejb() throws Exception {
        this.openDataProjects("TestApplication/TestApplication-ejb");
    }

    public void testOpenTestApplication_war() throws Exception {
        this.openDataProjects("TestApplication/TestApplication-war");
    }
}
