<%@page contentType="text/html" pageEncoding="UTF-8"%>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
   "http://www.w3.org/TR/html4/loose.dtd">

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
    <head>
        <title></title>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<%--CC
<style type="|"></style>
text/css
<style type="text/css"></style>
--%>


<style type="text/css">
    #pokus{
        background: green;
    }
    .prvniRadek{
        color:red;
    }
</style>


    </head>
    <body>
        <table id="smallTable">
            <tr class="firstLine">
                <td class="firstColumn"></td>
            </tr>
        </table>

<%--CC
<table id="|">
smallTable
<table id="smallTable">
--%>


<%--CC
<tr class="|">
prvniRadek
<tr class="prvniRadek">
--%>


    </body>
</html>
