
package testPackage;


/**
 * This is the local interface for ProductCodeTbl enterprise bean.
 */
public interface ProductCodeTblLocal extends javax.ejb.EJBLocalObject, testPackage.ProductCodeTblLocalBusiness {
    
    
}
