/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.websvc.api.jaxws.project.config;

/**
 *
 * @author Roderico Cruz
 */
public class WsimportOption {
   private org.netbeans.modules.websvc.jaxwsmodel.project_config1_0.WsimportOption wsimportOption;
   
   public WsimportOption(org.netbeans.modules.websvc.jaxwsmodel.project_config1_0.WsimportOption wsimportOption){
       this.wsimportOption = wsimportOption;
   }
   
   public String getWsimportOptionName(){
       return wsimportOption.getWsimportOptionName();
   }
   
   public void setWsimportOptionName(String name){
       wsimportOption.setWsimportOptionName(name);
   }
   
   public String getWsimportOptionValue(){
       return wsimportOption.getWsimportOptionValue();
   }
   
   public void setWsimportOptionValue(String value){
       wsimportOption.setWsimportOptionValue(value);
   }

   public Boolean getJaxbOption(){
       return wsimportOption.getJaxboption();
   }

   public void setJaxbOption(Boolean bool){
       wsimportOption.setJaxboption(bool);
   }
   public org.netbeans.modules.websvc.jaxwsmodel.project_config1_0.WsimportOption getOriginal(){
       return wsimportOption;
   }
}
