
package test;

import javax.ejb.EJBObject;


/**
 * This is the remote interface for EntityB enterprise bean.
 */
public interface EntityBRemote extends EJBObject, EntityBRemoteBusiness {
    
    
}
