root { 
    
/**CC
display: |
display block
display: block
*/

/**CC
color: |
color green
color: green
*/

/**CC
border-right: |
border-right inset
border-right: inset
*/

/**CC
border-bottom-color : |
border-bottom-color red
border-bottom-color : red
*/

}

