/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.dd.impl.web.metadata;

import java.util.ArrayList;
import java.util.List;
import org.netbeans.modules.j2ee.dd.api.web.WebApp;
import org.netbeans.modules.j2ee.dd.api.web.WebFragment;
import org.netbeans.modules.j2ee.dd.impl.web.annotation.AnnotationHelpers;

/**
 * @author Petr Slechta
 */
abstract class MergeEngine<T> {
    protected List<T> res = new ArrayList<T>();

    void clean() {
        res.clear();
    }

    abstract void addItems(WebApp webXml);
    abstract void addItems(WebFragment webFragment);
    abstract void addAnnotations(AnnotationHelpers annotationHelpers);

    List<T> getResult() {
        return res;
    }

    protected void addAll(T[] items) {
        if (items != null) {
            for (T item : items) {
                res.add(item);
            }
        }
    }
}
