/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.beans.api.model;

import java.util.Collection;

import org.netbeans.modules.web.beans.model.spi.WebBeansModelProvider;
import org.netbeans.modules.web.beans.model.spi.WebBeansModelProviderFactory;
import org.openide.util.Lookup;


/**
 * @author ads
 *
 */
public abstract class AbstractModelImplementation {
    
    protected AbstractModelImplementation( ModelUnit unit ){
        myUnit = unit;
        myModel = new WebBeansModel( this );
        Collection<? extends WebBeansModelProviderFactory> factories = 
            Lookup.getDefault().lookupAll( 
                WebBeansModelProviderFactory.class);
        for( WebBeansModelProviderFactory factory : factories ){
            myProvider = factory.createWebBeansModelProvider(this);
            if ( myProvider != null ){
                break;
            }
        }
        
    }
    
    public ModelUnit getModelUnit(){
        return myUnit;
    }
    
    public BeansModel getBeansModel(){
        return BeansModelFactory.getModel(getModelUnit());
    }

    protected WebBeansModel getModel(){
        return myModel;
    }
    
    protected WebBeansModelProvider getProvider(){
        return myProvider;
    }
    
    private ModelUnit myUnit;
    private WebBeansModel myModel;
    private WebBeansModelProvider myProvider;
}
