<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8"%>
<%--
The taglib directive below imports the JSTL library. If you uncomment it,
you must also add the JSTL library to the project. The Add Library... action
on Libraries node in Projects view can be used to add the JSTL 1.1 library.
--%>

<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%> 
<%@taglib uri="http://java.sun.com/jsf/html" prefix="h"%> 

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">

<%-- completion for jsp tags --%>
<%--CC
<jsp:useBean id="bean" scope="application" class="java.lang.Date"></|
...
<jsp:useBean id="bean" scope="application" class="java.lang.Date"></jsp:useBean>
--%>

<%-- completion for jsp tags --%>
<%--CC
<jsp:include page="index.jsp" flush="false"></jsp:i|
...
<jsp:include page="index.jsp" flush="false"></jsp:include>
--%>

<%-- completion for core lib tags --%>
<%--CC
<c:when test="" ></|
...
<c:when test="" ></c:when>
--%>

<%-- completion for core lib tags --%>
<%--CC
<c:forTokens delims=":;," items="items"></|
...
<c:forTokens delims=":;," items="items"></c:forTokens>
--%>


<%-- completion for html lib tags --%>
<%--CC
<h:inputTextarea ></|
...
<h:inputTextarea ></h:inputTextarea>
--%>

<%-- completion for html lib tags --%>
<%--CC
<h:outputFormat></h:out|
...
<h:outputFormat></h:outputFormat>
--%>

