/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.core.spi;

import org.netbeans.modules.web.core.api.ErrorInfo;
import org.openide.filesystems.FileObject;
import org.openide.util.Lookup;

/**
 *
 * @author marekfukala
 */
public interface ErrorAnnotationFactory {

    public ErrorAnnotation create(FileObject file);

    public static class Query {

        private static final ErrorAnnotation NULL_ERROR_ANNOTATION_IMPL = new NullErrorAnnotation();

        public static ErrorAnnotation create(FileObject file) {
            ErrorAnnotationFactory factory = Lookup.getDefault().lookup(ErrorAnnotationFactory.class);
            if(factory == null) {
                //this may happen since the method is called before the actual installation of the service implementation!
                return NULL_ERROR_ANNOTATION_IMPL;
            } else {
                return factory.create(file);
            }
        }

        private static final class NullErrorAnnotation implements ErrorAnnotation {
            @Override
            public void annotate(ErrorInfo[] errors) {
                //no-op
            }
        }
        
    }

}
