/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.maven.j2ee.execution;

/**
 * Constants related to the execution.
 *
 * @author Martin Janicek <mjanicek@netbeans.org>
 */
public final class ExecutionConstants {

    private ExecutionConstants() {
    }

    /**
     * Used to skip the build before deployment.
     */
    public static final String SKIP_BUILD = "skip.build"; // NOI18N

    /**
     * Used to perform standard execution (without application server startup etc.).
     */
    public static final String STANDARD_EXECUTION = "standard.execution"; // NOI18N
}
