
package testPackage;


/**
 * This is the local interface for SalesRepDataTbl enterprise bean.
 */
public interface SalesRepDataTblLocal extends javax.ejb.EJBLocalObject, testPackage.SalesRepDataTblLocalBusiness {
    
    
}
