<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8"%>


<%@taglib prefix="f" uri="http://java.sun.com/jsf/core"%>
<%@taglib prefix="h" uri="http://java.sun.com/jsf/html"%>



<%--CC
<h:inputText value="#{|}"/>
cookie
<h:inputText value="#{cookie}"/>
--%>


<%--CC
<h:outputText value="#{|}" title="Hi" />
MyBean MyBean
<h:outputText value="#{MyBean}" title="Hi" />
--%>

<%--CC
<h:outputText value="#{MyBean.n|}" title="Hi" />
name
<h:outputText value="#{MyBean.name}" title="Hi" />
--%>

