/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.j2ee.dd.api.web;

/**
 *	This generated bean class AbsoluteOrderingType matches the schema element 'absoluteOrderingType'.
 *  The root bean class is WebApp
 */
public interface AbsoluteOrdering {

	void setName(String[] value);
	String[] getName();
//	void setOthers(int index, OrderingOthersType value);
//	OrderingOthersType getOthers(int index);
//	int sizeOthers();
//	void setOthers(OrderingOthersType[] value);
//	OrderingOthersType[] getOthers();
//	int addOthers(org.netbeans.modules.j2ee.dd.impl.web.model_3_0.OrderingOthersType value);
//	int removeOthers(org.netbeans.modules.j2ee.dd.impl.web.model_3_0.OrderingOthersType value);
//	OrderingOthersType newOrderingOthersType();

}
