/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.jsfapi.api;

import org.junit.Assert;
import org.junit.Test;

/**
 *
 * @author marekfukala
 */
public class AttributeTest {
    
    public AttributeTest() {
    }

    @Test
    public void testDefaultAttribute() {
        String ms = "void validate(javax.faces.context.FacesContext)";
        Attribute a = new Attribute.DefaultAttribute(
                "attrname", "descr", "java.lang.Object", true, 
                ms);

        Assert.assertNotNull(a.getName());
        Assert.assertNotNull(a.getDescription());
        Assert.assertNotNull(a.getType());
        Assert.assertTrue(a.isRequired());
        Assert.assertEquals(ms, a.getMethodSignature());
        Assert.assertNull(a.getDefaultValue());
    }

    @Test
    public void testAttributeDefaultValue() {
        String ms = "void validate(javax.faces.context.FacesContext)";
        String value = "myValue";
        Attribute a = new Attribute.DefaultAttribute(
                "attrname", "descr", "java.lang.Object", true,
                ms, value);

        Assert.assertNotNull(a.getName());
        Assert.assertNotNull(a.getDescription());
        Assert.assertNotNull(a.getType());
        Assert.assertTrue(a.isRequired());
        Assert.assertEquals(ms, a.getMethodSignature());
        Assert.assertEquals(value, a.getDefaultValue());
    }
}
