<%@taglib uri="http://java.sun.com/jstl/core" prefix="c" %>
            <form action="j_security_check" method="POST">
             Username:<input type="text" name="j_username"><br>
             Password:<input type="password" name="j_password">
                <input type="submit" value="Login">
            </form>
