
package testPackage;


/**
 * This is the local interface for OfficeTbl enterprise bean.
 */
public interface OfficeTblLocal extends javax.ejb.EJBLocalObject, testPackage.OfficeTblLocalBusiness {
    
    
}
