<?php
/* 
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Description of PHPClass
 *
 * @author KAM
 */
class PHPClass {
    //put your code here
}
?>
