
package testPackage;


/**
 * This is the local interface for ProductTbl enterprise bean.
 */
public interface ProductTblLocal extends javax.ejb.EJBLocalObject, testPackage.ProductTblLocalBusiness {
    
    
}
