<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8"%>
<h3>incl/embeddedDynamicInclude.jsp</h3>

<jsp:include page="simpleDynamicInclude.jsp"/>
<jsp:include page="../incl/simpleDynamicInclude.jsp"/>