<table width='75%' align='CENTER' border=0 height='75%'>
<tr valign='top' >
    <td align='center'>
        <div class="subtitle">
            <h1>SecurityFilter Example Application: Login Error Page</h1>
            Bad username/password combination, please try again.
            <p>Request a secure Admin page <a href="secureAdmin/secureP.html">here!</a></p>
            <p>Request a secure User page <a href="secureUser/secureP.html">here!</a></p>

        </div>
    </td>
</tr>
</table>