/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.web.beans.impl.model;

import org.netbeans.modules.web.beans.api.model.AbstractModelImplementation;
import org.netbeans.modules.web.beans.model.spi.WebBeansModelProvider;
import org.netbeans.modules.web.beans.model.spi.WebBeansModelProviderFactory;


/**
 * @author ads
 *
 */
@org.openide.util.lookup.ServiceProvider(service=WebBeansModelProviderFactory.class)
public class WebBeansProviderFactoryImpl implements
        WebBeansModelProviderFactory
{

    /* (non-Javadoc)
     * @see org.netbeans.modules.web.beans.model.spi.WebBeansModelProviderFactory#createWebBeansModelProvider(org.netbeans.modules.web.beans.api.model.AbstractModelImplementation)
     */
    @Override
    public WebBeansModelProvider createWebBeansModelProvider(
            AbstractModelImplementation model )
    {
        if ( model instanceof WebBeansModelImplementation ){
            return new WebBeansModelProviderImpl( 
                    (WebBeansModelImplementation)model );
        }
        return null;
    }

}
