/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.earproject.ui.wizards;

import java.io.File;
import javax.swing.JPanel;
import javax.swing.event.DocumentListener;
import org.openide.WizardDescriptor;

abstract class SettingsPanel extends JPanel {

    abstract void store (WizardDescriptor settings);

    abstract void read (WizardDescriptor settings);

    abstract boolean valid (WizardDescriptor settings);
    
    abstract void addNameListener(DocumentListener dl);
    
    protected static File findExistingParent(String path) {
        File ret = new File(path).getParentFile();
        while (ret != null && !ret.exists()) {
            ret = ret.getParentFile();
        }
        return ret;
    }
    
    static boolean isValidProjectName(String projectName) {
        return projectName.length() != 0
                && projectName.indexOf('*')  < 0
                && projectName.indexOf('/')  < 0
                && projectName.indexOf('\\') < 0
                && projectName.indexOf(':')  < 0;
    }

}
