/*
 * TmpI.java
 *
 * Created on July 22, 2005, 1:12 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package org.netbeans.test.j2ee.tmp;

/**
 *
 * @author jungi
 */
public interface TmpI {
    
    String toString();
}
