/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.j2ee.ejbcore;

import org.netbeans.api.project.Project;
import org.netbeans.modules.j2ee.api.ejbjar.EjbJar;
import org.netbeans.modules.j2ee.ejbcore.ui.logicalview.ejb.entity.EntityNode;
import org.netbeans.modules.j2ee.ejbcore.ui.logicalview.ejb.mdb.MessageNode;
import org.netbeans.modules.j2ee.ejbcore.ui.logicalview.ejb.session.SessionNode;
import org.netbeans.modules.j2ee.spi.ejbjar.EjbNodesFactory;
import org.openide.nodes.Node;

/**
 *
 * @author Pavel Buzek
 */
@org.openide.util.lookup.ServiceProvider(service=org.netbeans.modules.j2ee.spi.ejbjar.EjbNodesFactory.class)
public final class EjbNodesFactoryImpl implements EjbNodesFactory {
    
    public EjbNodesFactoryImpl() {
    }
    
    public Node createSessionNode(String ejbClass, EjbJar ejbModule, Project project) {
        return SessionNode.create(ejbClass, ejbModule, project);
    }
    
    public Node createEntityNode(String ejbClass, EjbJar ejbModule, Project project) {
        return EntityNode.create(ejbClass, ejbModule, project);
    }
    
    public Node createMessageNode(String ejbClass, EjbJar ejbModule, Project project) {
        return MessageNode.create(ejbClass, ejbModule, project);
    }
}
