<%@page contentType="text/html" pageEncoding="US-ASCII"%>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
   "http://www.w3.org/TR/html4/loose.dtd">

<html>
    <head>
        <title></title>
        <meta http-equiv="Content-Type" content="text/html; charset=US-ASCII">
        <style type ="text/css">
            h1{
                color: green;
            }
        </style>
    </head>
    <body>
        <table style="text-align: center; border: 1px ">
            <tr>
                <td></td>
                <td></td>
            </tr>
            <tr>
                <td></td>
                <td>
            </tr>
        </table>
        <%! 
        int count = 10;
        %>
        <%
        if (true){
            for (int i = 0; i< count; i++){
                
            }
        }
        
        %>
    </body>
</html>
