package sample;

import java.rmi.RemoteException;


/**
 * This is the implementation bean class for the SampleWebService web service.
 * Created 2.6.2005 15:26:52
 * @author lm97939
 */
public class SampleWebServiceImpl implements SampleWebServiceSEI {
    
    
    // Enter web service operations here. (Popup menu: Web Service->Add Operation)
    /**
     * Web service operation
     */
    public String operation1() throws RemoteException {
        // TODO implement operation 
        return null;
    }

    /**
     * Web service operation
     */
    public String operation2(String a, int b) throws java.lang.Exception, java.rmi.RemoteException {
        // TODO implement operation 
        return null;
    }
}
