package applicationclient1;

import junit.framework.TestCase;

public class NewClassTest extends TestCase {
    
    public NewClassTest(String testName) {
        super(testName);
    }

    public void testMethod() {
    }
    
}
