/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.web.jsf.impl.facesmodel;

import org.netbeans.modules.web.jsf.api.facesmodel.*;
import org.w3c.dom.Element;

/**
 *
 * @author Petr Pisl
 */
public class DisplayNameImpl extends JSFConfigComponentImpl implements DisplayName{

    public DisplayNameImpl(JSFConfigModelImpl model, Element element) {
        super(model, element);
    }
    
    public DisplayNameImpl(JSFConfigModelImpl model) {
        super(model, createElementNS(model, JSFConfigQNames.DISPLAY_NAME));
    }

    public void accept(JSFConfigVisitor visitor) {
        visitor.visit(this);
    }

    public String getValue() {
        return getText();
    }

    public void setValue(String displayName) {
        setText(DescriptionGroup.DISPLAY_NAME, displayName);
    }

    public String getLang() {
        return getAttribute(FacesAttributes.LANG);
    }

    public void setLang(String lang) {
        setAttribute(LangAttribute.LANG_ATTRIBUTE, FacesAttributes.LANG, lang);
    }

}
