
function myfunction(){
    
}

var myvariable="variable";

x = "foo";
y = x;

/**CC
y.cha
String charCodeAt ( index ) : Number
y.charCodeAt(index)
*/

/**CC
de|
debugger
debugger
*/

/**CC
va|
var
var
*/

/**CC
St|
Core JS String
String
*/

/**CC #99811
my|
testJS.js myfunction () : void
myfunction()
*/

/**CC #112683
my|
testJS.js myvariable
myvariable
*/

/**CC
Ra|
Core JS RangeError
RangeError
*/

