/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.netbeans.modules.websvc.design.view.widget;

import java.awt.Image;
import org.netbeans.api.visual.widget.Widget;

/**
 *
 * @author Ajit
 */
public interface TabWidget {
    
    /**
     * Return the title to be displayed in this tab.
     * @return title the title to be displayed in this tab.
     */ 
    String getTitle();
    
    /**
     * Return the image icon to be displayed in this tab.
     * @return image the image icon to be displayed in this tab.
     */ 
    Image getIcon();
    
    /**
     * Return the widget to be displayed in this tab.
     * @return widget the widget to be displayed in this tab.
     */ 
    Widget getComponentWidget();
    
    /**
     * Returns the object that can be used as a hashtable key.
     * @return  hashtable key.
     */
    Object hashKey();
}
