/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.j2ee.ddloaders.catalog;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import org.netbeans.core.startup.Main;
import org.netbeans.junit.NbTestCase;

/**
 * Test for {@link EnterpriseCatalog}
 *
 * @author Pete Whelpton
 */
public class EnterpriseCatalogTest extends NbTestCase {

    public EnterpriseCatalogTest(String name) {
        super(name);
    }

    @Override
    protected void setUp() throws Exception {
        Main.initializeURLFactory();
        super.setUp();
    }

    public void testSchemasExist() {

        EnterpriseCatalog catalog = new EnterpriseCatalog();
        Iterator publicIDs = catalog.getPublicIDs();

        boolean pass = true;

        while (publicIDs.hasNext()) {
            String publicID = publicIDs.next().toString();
            String systemID = catalog.getSystemID(publicID);

            try {
                URL url = new URL(systemID);
                url.openConnection().connect();
            } catch (MalformedURLException ex) {
                System.out.println("Invalid URL " + systemID);
                pass = false;
            } catch (IOException ex) {
                System.out.println("Could not find schema " + systemID);
                pass = false;
            }
        }

        assertTrue(pass);
    }

}
