/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
/*
 * SessionConfig.java
 *
 * Created on November 15, 2004, 4:26 PM
 */

package org.netbeans.modules.j2ee.sun.dd.api.web;

public interface SessionConfig extends org.netbeans.modules.j2ee.sun.dd.api.CommonDDBean {

        public static final String SESSION_MANAGER = "SessionManager";	// NOI18N
	public static final String SESSION_PROPERTIES = "SessionProperties";	// NOI18N
	public static final String COOKIE_PROPERTIES = "CookieProperties";	// NOI18N

        /** Setter for session-manager property
         * @param value property value
         */
	public void setSessionManager(SessionManager value); 
        /** Getter for session-manager property.
         * @return property value
         */
	public SessionManager getSessionManager();

	public SessionManager newSessionManager();
        /** Setter for session-properties property
         * @param value property value
         */
	public void setSessionProperties(SessionProperties value); 
        /** Getter for session-properties property.
         * @return property value
         */
	public SessionProperties getSessionProperties();

	public SessionProperties newSessionProperties();
        /** Setter for cookie-properties property
         * @param value property value
         */
	public void setCookieProperties(CookieProperties value);
        /** Getter for cookie-properties property.
         * @return property value
         */
	public CookieProperties getCookieProperties();

	public CookieProperties newCookieProperties(); 

}
