<%@page contentType="text/html"%>
<%@page pageEncoding="UTF-8"%>

<%-- completion for HTML tags --%>
<%--CC
<h|
html
<html
--%>

<%-- completion for HTML attributes --%>
<%--CC
<a |
href
<a href=""
--%>

<%-- completion for HTML values --%>
<%--CC
<p dir="|
rtl
<p dir="rtl
--%>

<%-- completion for body HTML tag --%>
<%--CC
<b|
body
<body
--%>

<%-- meta should not be offered --%>
<%--CC
<head><meta></|
/head
<head><meta></head>
--%>

<html>
    <head><title>HTML Code Completion Page</title></head>
    <body>
        <h1>HTML Code Completion Page</h1>
    </body>
</html>
